drop table if exists construct;
create table construct (
	id int not null primary key auto_increment,
	name_id int not null,
	ml_id int not null default 1,
	keyword char(64) not null,
	childable tinyint not null,
	intext tinyint not null,
	locked int not null,
	locked_by int not null,
	global int not null,
	icon char(64) not null,
	creator int not null,
	cdate int not null,
	editor int not null,
	edate int not null,
	description_id int not null
);

drop table if exists construct_node;
create table construct_node (
	construct_id int not null,
	node_id int not null
) type=InnoDB;

drop table if exists contentfile;
create table contentfile (
	id int not null primary key auto_increment,
	name char(255) not null,
	filetype char(64) not null,
	folder_id int not null,
	filesize int not null,
	creator int not null,
	cdate int not null,
	editor int not null,
	edate int not null,
	description char(255) not null,
  sizex int not null,
  sizey int not null
)type=InnoDB;

drop table if exists ml;
create table ml (
	id int not null primary key auto_increment,
	name char(255) not null,
	ext char(8) not null
)type=InnoDB;

drop table if exists type;
create table type (
	id int not null primary key auto_increment,
	name char(255) not null,
	description char(255) not null,
	auto tinyint not null
)type=InnoDB;

drop table if exists part;
create table part (
	id int not null primary key auto_increment,
	construct_id int not null,
	type_id int not null,
	name_id int not null,
	required tinyint not null,
	editable tinyint not null,
	partoption_id int not null,
	partorder smallint not null default 1,
	keyword char(64) not null,
	hidden tinyint not null
)type=InnoDB;

drop table if exists value;
create table value (
	id int not null primary key auto_increment,
	part_id int not null,
	info int not null,
	static tinyint not null,
	templatetag_id int not null,
	contenttag_id int not null,
	globaltag_id int not null,
	objtag_id int not null,
	value_text mediumtext not null,
	value_ref int not null
) type=InnoDB;

drop table if exists globaltag;
create table globaltag (
	id int not null primary key auto_increment,
	construct_id int not null,
	name char(255) not null,
	creator int not null,
	cdate int not null,
	editor int not null,
	edate int not null,
	description char(255) not null,
	enabled tinyint not null
)type=InnoDB;

drop table if exists templatetag;
create table templatetag (
	id int not null primary key auto_increment,
	templategroup_id int not null,
	template_id int not null,
	construct_id int not null default 1,
	pub tinyint not null,
	enabled tinyint not null,
	name char(255) not null
) type=InnoDB;

drop table if exists contenttag;
create table contenttag (
	id int not null primary key auto_increment,
	content_id int not null,
	construct_id int not null,
	enabled tinyint not null,
	name char(127) not null
) type=InnoDB;

drop table if exists contenttag_nodeversion;
create table contenttag_nodeversion (
	id int not null,
	content_id int not null,
	construct_id int not null,
	enabled tinyint not null,
	name char(127) not null,
	nodeversiontimestamp int not null,
	nodeversion_user int not null,
	nodeversionlatest int not null,
	nodeversionremoved int not null
) type=InnoDB;

drop table if exists objtag;
create table objtag (
	id int not null primary key auto_increment,
	obj_id int not null,
	obj_type int not null,
	construct_id int not null,
	enabled tinyint not null,
	name char(255) not null,
	required int not null default 0,
	inpage int not null default 0,
	intag int not null,
	inheritable int not null default 0
) type=InnoDB;

drop table if exists globaltag_node;
create table globaltag_node (
	globaltag_id int not null,
	node_id int not null
) type=InnoDB;

drop table if exists folder;
create table folder (
	id int not null primary key auto_increment,
	mother int not null,
	name char(255) not null,
	type_id smallint not null,
	pub_dir char(255) not null,
	node_id int not null,
	creator int not null,
	cdate int not null,
	editor int not null,
	edate int not null,
	description char(255) not null
) type=InnoDB;

drop table if exists node;
create table node (
	id int not null primary key auto_increment,
	folder_id int not null,
	pub_dir char(255) not null,
	host char(255) not null,
	ftphost char(255) not null,
	ftplogin char(255) not null,
	ftppassword char(255) not null,
	ftpsync tinyint not null,
    ftpwwwroot char(255) not null,
	creator int not null,
	cdate int not null,
	editor int not null,
	edate int not null,
	utf8 int not null,
  publish_fs int not null,
  publish_contentmap int not null
) type=InnoDB;

drop table if exists template;
create table template (
	id int not null primary key auto_increment,
	folder_id int not null,
	templategroup_id int not null,
	name char(255) not null,
	locked int not null,
	locked_by int not null,
	ml_id int not null,
	ml mediumtext not null,
	creator int not null,
	cdate int not null,
	editor int not null,
	edate int not null,
	description char(255) not null
) type=InnoDB;

drop table if exists templategroup;
create table templategroup (
	id int not null primary key auto_increment,
	nada tinyint not null
) type=InnoDB;

drop table if exists template_folder;
create table template_folder (
	template_id int not null,
	folder_id int not null
) type=InnoDB;

drop table if exists page;
create table page (
	id int not null primary key auto_increment,
	name char(255) not null,
	description char(255) not null,
	filename char(64) not null,
	priority tinyint not null default 1,
	status tinyint not null,
	time_start int not null,
	time_end int not null,
	time_mon tinyint not null,
	time_tue tinyint not null,
	time_wed tinyint not null,
	time_thu tinyint not null,
	time_fri tinyint not null,
	time_sat tinyint not null,
	time_sun tinyint not null,
	time_pub tinyint not null,
	content_id int not null,
	template_id int not null,
	folder_id int not null,
	creator int not null,
	cdate int not null,
	editor int not null,
	edate int not null,
	pdate int not null,
	publisher int not null
	contentgroup_id int not null,
	contentset_id int not null
) type=InnoDB;

drop table if exists content;
create table content (
	id int not null primary key auto_increment,
	node_id int not null,
	locked int not null,
	locked_by int not null,
	creator int not null,
	cdate int not null,
	editor int not null,
	edate int not null
) type=InnoDB;

drop table if exists contenttable;
create table contenttable (
	templatetag_id int not null,
	contenttag_id int not null,
	globaltag_id int not null,
	objtag_id int not null,
	part_id int not null,
	th tinyint not null,
	td_f_even text not null,
	td_f_odd text not null,
	th_f text not null,
	td_type int not null,
	th_type int not null,
	table_f text not null
) type=InnoDB;

drop table if exists ds;
create table ds (
	id int not null primary key auto_increment,
	templatetag_id int not null,
	contenttag_id int not null,
	globaltag_id int not null,
	objtag_id int not null,
	o_type int not null,
	is_folder tinyint not null,
	orderkind tinyint not null,
	orderway tinyint not null,
	max_obj int not null,
	recursion tinyint not null
) type=InnoDB;

drop table if exists ds_nodeversion;
create table ds_nodeversion (
	id int not null,
	templatetag_id int not null,
	contenttag_id int not null,
	globaltag_id int not null,
	objtag_id int not null,
	o_type int not null,
	is_folder tinyint not null,
	orderkind tinyint not null,
	orderway tinyint not null,
	max_obj int not null,
	recursion tinyint not null,
	nodeversiontimestamp int not null,
	nodeversion_user int not null,
	nodeversionlatest int not null,
	nodeversionremoved int not null
) type=InnoDB;

drop table if exists ds_obj;
create table ds_obj (
	id int not null primary key auto_increment,
	ds_id int not null,
	templatetag_id int not null,
	contenttag_id int not null,
	globaltag_id int not null,
	objtag_id int not null,
	o_id int not null,
	obj_order smallint not null,
	auser int not null,
	adate int not null
) type=InnoDB;

drop table if exists ds_obj_nodeversion;
create table ds_obj_nodeversion (
	id int not null,
	ds_id int not null,
	templatetag_id int not null,
	contenttag_id int not null,
	globaltag_id int not null,
	objtag_id int not null,
	o_id int not null,
	obj_order smallint not null,
	nodeversiontimestamp int not null,
	nodeversion_user int not null,
	nodeversionlatest int not null,
	nodeversionremoved int not null,
	auser int not null,
	adate int not null
) type=InnoDB;

drop table if exists publish;
create table publish (
	id int not null primary key auto_increment,
	page_id int not null,
	node_id int not null,
	folder_id int not null,
	path char(255) not null,
	filename char(255) not null,
	source mediumtext not null,
	pdate int not null,
	ddate int not null,
	active tinyint not null
) type=InnoDB;

drop table if exists objtranslatecache;
create table objtranslatecache (
	o_id int not null,
	o_type int not null,
	cachekey char(255) not null,
	value_tags mediumtext not null
) type=InnoDB;

drop table if exists contentgroup;
create table contentgroup (
	id int not null primary key auto_increment,
	name char(255) not null,
	code char(5) not null
) type=InnoDB;

drop table if exists contentset;
create table contentset (
	id int not null primary key auto_increment,
	nada tinyint not null
) type=InnoDB;

drop table if exists cnstatconfig;
create table cnstatconfig (
	id int not null primary key auto_increment,
	name varchar(255) not null,
	statkeys text not null,
	active int not null
) type=InnoDB;

drop table if exists cnstatval;
create table cnstatval (
	id int not null primary key auto_increment,
	cnstatconfig_id int not null,
	timestamp int not null,
	field varchar(255) not null,
	col int not null,
	textvalue text not null,
	intvalue int not null
) type=InnoDB;

drop table if exists cn_map;
create table cn_map (
    id int not null primary key auto_increment,
    name char(255) not null,
    keyword char(255) not null,
    description char(255) not null,
    open_target char(255) null,
    page_id int not null,
    is_link_int tinyint null,
    node_id int not null,
    mother_id int not null,
    creator int not null,
    cdate int not null,
    editor int not null,
    edate int not null
) type=InnoDB;

drop table if exists datasource;
create table datasource ( 
	id int not null primary key auto_increment, 
	source_type int not null, 
	name varchar(255) 
) type=InnoDB;

drop table if exists datasource_value;
create table datasource_value( 
	id int not null primary key auto_increment, 
	datasource_id int not null,
	sorder int,
    dskey varchar(255), 
	value text 
) type=InnoDB;

create table value (
	id int not null primary key auto_increment,
	part_id int not null,
	info int not null,
	static tinyint not null,
	templatetag_id int not null,
	contenttag_id int not null,
	globaltag_id int not null,
	objtag_id int not null,
	value_text mediumtext not null,
	value_ref int not null
) type=InnoDB;

drop table if exists value_nodeversion;
create table value_nodeversion (
	id int not null,
	part_id int not null,
	info int not null,
	static tinyint not null,
	templatetag_id int not null,
	contenttag_id int not null,
	globaltag_id int not null,
	objtag_id int not null,
	value_text mediumtext not null,
	value_ref int not null,
	nodeversiontimestamp int not null,
	nodeversion_user int not null,
	nodeversionlatest int not null,
	nodeversionremoved int not null
) type=InnoDB;

drop table if exists nodeversion;
create table nodeversion (
	id int not null,
	timestamp int not null,
	user_id int not null,
	o_type int not null,
	o_id int not null,
	nodeversion decimal(8,3) not null
) type=InnoDB;

drop table if exists node_contentgroup;
create table node_contentgroup (
	node_id int not null,
	contentgroup_id int not null,
	sortorder int not null
) type=InnoDB;

drop table if exists cn_mapprop;
create table cn_mapprop ( 
	id int not null primary key auto_increment, 
	cn_map_id int not null, 
	ptype int,
	pkey varchar(100), 
	value text 
) type=InnoDB;

create table cn_folder_map (
	folder_id int not null, 
	cn_map_id int not null, 
	primary key(folder_id, cn_map_id)
) type = InnoDB;


drop table if exists import_page;
create table import_page (
	id int not null primary key auto_increment,
	import_id int not null,
	page_id int not null,
	url varchar(255) not null,
	type char(5) not null
);

drop table if exists import_part;
create table import_part (
	id int not null primary key auto_increment,
	import_page_id int not null,
	tag_id int not null,
	part_id int not null,
	type_id int not null
);

drop table if exists tagmap;
create table tagmap (
	tagname char(255) not null,
	mapname char(255) not null,
	searchname char(255) not null,
	object int not null,
	objtype int not null,
	attributetype int not null,
	multivalue int(1) not null,
	static int not null,
	optimized int not null
);

drop table if exists contentmap;
CREATE TABLE contentmap (
  contentid char(32) NOT NULL primary key default '',
  obj_id int(11) NOT NULL default '0',
  obj_type int(11) NOT NULL default '0',
  motherid char(32) not null,
  mother_obj_id int(11) NOT NULL default '0',
  mother_obj_type int(11) NOT NULL default '0',
  updatetimestamp int(11) NOT NULL default '0'
) TYPE=InnoDB;


drop table if exists contentattribute;
CREATE TABLE contentattribute (
  contentid varchar(32) NOT NULL default '',
  name varchar(255) NOT NULL default '',
  value_text mediumtext,
  value_bin mediumblob,
  value_int int(11) default NULL,
  sortorder int(11) default NULL
) TYPE=InnoDB;

drop table if exists contentattributetype;
create table contentattributetype (
	name char(255),
	attributetype int,
	optimized int,
	quickname char(255)
) type=InnoDB;

drop table if exists contentstatus;
create table contentstatus (
	name char(255) primary key,
	intvalue int(11)
) type=InnoDB;

drop table if exists param;
create table param ( 
	paramid int, 
	name varchar(30), 
	value text 
) type=InnoDB;
