<?php
//
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//        //           /       /                                           
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//
//                                                     
//
//                                                                   
//                          
//

//
//                                                                     
//                                                                    
//                                                               
//


require_once(INCLUDE_PATH."/common.php");

class DB_oci8 extends DB_common
{
//               

var $connection;
var $phptype, $dbsyntax;
var $manip_query = array();
var $prepare_types = array();
var $autoCommit = 1;
var $last_stmt = false;

//    
 //                

function DB_oci8()
{
$this->DB_common();
$this->phptype = 'oci8';
$this->dbsyntax = 'oci8';
$this->features = array(
'prepare' => false,
'pconnect' => true,
'transactions' => true,
'limit' => 'alter'
);
$this->errorcode_map = array(
900 => DB_ERROR_SYNTAX,
904 => DB_ERROR_NOSUCHFIELD,
923 => DB_ERROR_SYNTAX,
942 => DB_ERROR_NOSUCHTABLE,
955 => DB_ERROR_ALREADY_EXISTS,
1476 => DB_ERROR_DIVZERO,
1722 => DB_ERROR_INVALID_NUMBER,
2289 => DB_ERROR_NOSUCHTABLE,
2291 => DB_ERROR_CONSTRAINT,
2449 => DB_ERROR_CONSTRAINT,
);
}

//    
 //              

/**
     *                                                        
     *
     *                                                               
     *                                                            
     *                     
     *
     *                                                         
     */
function connect($dsninfo, $persistent = false)
{
if (!DB::assertExtension('oci8')) {
return $this->raiseError(DB_ERROR_EXTENSION_NOT_FOUND);
}
$this->dsn = $dsninfo;
$user = $dsninfo['username'];
$pw = $dsninfo['password'];
$hostspec = $dsninfo['hostspec'];

$connect_function = $persistent ? 'OCIPLogon' : 'OCILogon';

if ($hostspec) {
$conn = @$connect_function($user,$pw,$hostspec);
} elseif ($user || $pw) {
$conn = @$connect_function($user,$pw);
} else {
$conn = false;
}
if ($conn == false) {
$error = OCIError();
$error = (is_array($error)) ? $error['message'] : null;
return $this->raiseError(DB_ERROR_CONNECT_FAILED, null, null,
null, $error);
}
$this->connection = $conn;
return DB_OK;
}

//    
 //                 

/**
     *                                          
     *
     *                                                      
     */
function disconnect()
{
$ret = @OCILogOff($this->connection);
$this->connection = null;
return $ret;
}

//    
 //                  

/**
     *                                                                  
     *            
     *
     *                            
     *
     *                                                              
     *                                                              
     *                        
     */
function simpleQuery($query)
{
$this->last_query = $query;
$query = $this->modifyQuery($query);
$result = @OCIParse($this->connection, $query);
if (!$result) {
return $this->oci8RaiseError();
}
if ($this->autoCommit) {
$success = @OCIExecute($result,OCI_COMMIT_ON_SUCCESS);
} else {
$success = @OCIExecute($result,OCI_DEFAULT);
}
if (!$success) {
return $this->oci8RaiseError($result);
}
$this->last_stmt=$result;
//                                                           
 //                                  
 return DB::isManip($query) ? DB_OK : $result;
}

//    
 //                 

/**
     *                                                                     
     *
     *                                    
     *
     *               
     *
     *                                                             
     */
function nextResult($result)
{
return false;
}

//    
 //               

/**
     *                                 
     *
     *                                      
     *                                                            
     *
     *                                                                  
     *                                     
     */
function &fetchRow($result, $fetchmode = DB_FETCHMODE_DEFAULT)
{
if ($fetchmode == DB_FETCHMODE_DEFAULT) {
$fetchmode = $this->fetchmode;
}
if ($fetchmode & DB_FETCHMODE_ASSOC) {
$moredata = @OCIFetchInto($result, $row, OCI_ASSOC + OCI_RETURN_NULLS + OCI_RETURN_LOBS);
} else {
$moredata = @OCIFetchInto($result, $row, OCI_RETURN_NULLS + OCI_RETURN_LOBS);
}
if (!$moredata) {
return NULL;
}
return $row;
}

//    
 //                

/**
     *                                                        
     *
     *                                      
     *                                                                
     *                                                       
     *                                                           
     *
     *                                                         
     */
function fetchInto($result, &$arr, $fetchmode = DB_FETCHMODE_DEFAULT, $rownum=NULL)
{
if ($rownum !== NULL) {
return $this->raiseError(DB_ERROR_NOT_CAPABLE);
}
if ($fetchmode & DB_FETCHMODE_ASSOC) {
$moredata = @OCIFetchInto($result,$arr,OCI_ASSOC+OCI_RETURN_NULLS+OCI_RETURN_LOBS);
if ($moredata && $this->options['optimize'] == 'portability') {
$arr = array_change_key_case($arr, CASE_LOWER);
}
} else {
$moredata = @OCIFetchInto($result,$arr,OCI_RETURN_NULLS+OCI_RETURN_LOBS);
}
if (!$moredata) {
return NULL;
}
return DB_OK;
}

//    
 //                 

/**
     *                                                     
     *
     *                                                                 
     *
     *                                                          
     */
function freeResult($result)
{
if (is_resource($result)) {
return @OCIFreeStatement($result);
}
if (!isset($this->prepare_tokens[(int)$result])) {
return false;
}
unset($this->prepare_tokens[(int)$result]);
unset($this->prepare_types[(int)$result]);
unset($this->manip_query[(int)$result]);
return true;
}

//    
 //              

function numRows($result)
{
//                                   
 if ($this->options['optimize'] == 'portability' &&
$result === $this->last_stmt) {
$countquery = "SELECT COUNT(*) FROM (".$this->last_query.")";
$save_query = $this->last_query;
$save_stmt = $this->last_stmt;
$count = $this->query($countquery);
if (DB::isError($count) ||
DB::isError($row = $count->fetchRow(DB_FETCHMODE_ORDERED)))
{
$this->last_query = $save_query;
$this->last_stmt = $save_stmt;
return $this->raiseError(DB_ERROR_NOT_CAPABLE);
}
return $row[0];
}
return $this->raiseError(DB_ERROR_NOT_CAPABLE);
}

//    
 //              

/**
     *                                           
     *
     *                                      
     *
     *                                                     
     */
function numCols($result)
{
$cols = @OCINumCols($result);
if (!$cols) {
return $this->oci8RaiseError($result);
}
return $cols;
}

//    
 //                  

/**
     *                                                                  
     *                                                                 
     *                                                            
     *                         
     *
     *                                   
     */
function errorNative()
{
if (is_resource($this->last_stmt)) {
$error = @OCIError($this->last_stmt);
} else {
$error = @OCIError($this->connection);
}
if (is_array($error)) {
return $error['code'];
}
return false;
}

//    
 //              

/**
     *                                                              
     *                        
     *                                   
     *
     *                              
     */
function prepare($query)
{
$tokens = split('[\&\?]', $query);
$token = 0;
$types = array();
for ($i = 0; $i < strlen($query); $i++) {
switch ($query[$i]) {
case '?':
$types[$token++] = DB_PARAM_SCALAR;
break;
case '&':
$types[$token++] = DB_PARAM_OPAQUE;
break;
}
}
$binds = sizeof($tokens) - 1;
$newquery = '';
for ($i = 0; $i < $binds; $i++) {
$newquery .= $tokens[$i] . ":bind" . $i;
}
$newquery .= $tokens[$i];
$this->last_query = $query;
$newquery = $this->modifyQuery($newquery);
$stmt = @OCIParse($this->connection, $newquery);
$this->prepare_types[$stmt] = $types;
$this->manip_query[(int)$stmt] = DB::isManip($query);
return $stmt;
}

//    
 //              

/**
     *                                                 
     *
     *                                                               
     *                                                           
     *
     *                                                           
     *                                                                
     *                             
     */
function execute($stmt, $data = false)
{
$types=&$this->prepare_types[$stmt];
if (($size = sizeof($types)) != sizeof($data)) {
return $this->raiseError(DB_ERROR_MISMATCH);
}
for ($i = 0; $i < $size; $i++) {
if (is_array($data)) {
$pdata[$i] = &$data[$i];
}
else {
$pdata[$i] = &$data;
}
if ($types[$i] == DB_PARAM_OPAQUE) {
$fp = fopen($pdata[$i], "r");
$pdata[$i] = '';
if ($fp) {
while (($buf = fread($fp, 4096)) != false) {
$pdata[$i] .= $buf;
}
}
}
if (!@OCIBindByName($stmt, ":bind" . $i, $pdata[$i], -1)) {
return $this->oci8RaiseError($stmt);
}
}
if ($this->autoCommit) {
$success = @OCIExecute($stmt, OCI_COMMIT_ON_SUCCESS);
}
else {
$success = @OCIExecute($stmt, OCI_DEFAULT);
}
if (!$success) {
return $this->oci8RaiseError($stmt);
}
$this->last_stmt = $stmt;
if ($this->manip_query[(int)$stmt]) {
return DB_OK;
} else {
return new DB_result($this, $stmt);
}
}

//    
 //                 

/**
     *       /                         
     *
     *                   /                           
     */
function autoCommit($onoff = false)
{
$this->autoCommit = (bool)$onoff;;
return DB_OK;
}

//    
 //             

/**
     *                                              
     *
     *                          
     */
function commit()
{
$result = @OCICommit($this->connection);
if (!$result) {
return $this->oci8RaiseError();
}
return DB_OK;
}

//    
 //               

/**
     *                                                                  
     *
     *                          
     */
function rollback()
{
$result = @OCIRollback($this->connection);
if (!$result) {
return $this->oci8RaiseError();
}
return DB_OK;
}

//    
 //                   

/**
     *                                                    
     *                                           
     *
     *                                                              
     */
function affectedRows()
{
if ($this->last_stmt === false) {
return $this->oci8RaiseError();
}
$result = @OCIRowCount($this->last_stmt);
if ($result === false) {
return $this->oci8RaiseError($this->last_stmt);
}
return $result;
}

//    
 //                  

function modifyQuery($query)
{
//                                                      
 if (preg_match('/^\s*SELECT/i', $query) &&
!preg_match('/\sFROM\s/i', $query)) {
$query .= " FROM dual";
}
return $query;
}

//    
 //                       

/**
    *                                                 
    *
    *                                        
    *                                                    
    *                                
    *                                  
    *
    *                                        
    */
function modifyLimitQuery($query, $from, $count)
{
//                                                     
 //                           
 $q_fields = "SELECT * FROM ($query) WHERE NULL = NULL";
if (!$result = OCIParse($this->connection, $q_fields)) {
return $this->oci8RaiseError();
}
if (!OCIExecute($result, OCI_DEFAULT)) {
return $this->oci8RaiseError($result);
}
$ncols = OCINumCols($result);
$cols  = array();
for ( $i = 1; $i <= $ncols; $i++ ) {
$cols[] = OCIColumnName($result, $i);
}
$fields = implode(', ', $cols);
//                                
 //               /   *         /                       
 //    //                                                
 //                                                               
 //                     /*             */            
 // 

//                    
 //               //                     /                              
 //                                                       
 $from += 1; //                             
 $query = "SELECT $fields FROM".
"  (SELECT rownum as linenum, $fields FROM".
"      ($query)".
"  WHERE rownum <= ". ($from + $count) .
") WHERE linenum >= $from";
return $query;
}

//    
 //             

/**
     *                                                        
     *                                                           
     *
     *               
     *
     *                                          
     *
     *                                                                
     *                  
     *
     *                                          
     */
function nextId($seq_name, $ondemand = true)
{
$seqname = $this->getSequenceName($seq_name);
$repeat = 0;
do {
$this->expectError(DB_ERROR_NOSUCHTABLE);
$result = $this->query("SELECT ${seqname}.nextval FROM dual");
$this->popExpect();
if ($ondemand && DB::isError($result) &&
$result->getCode() == DB_ERROR_NOSUCHTABLE) {
$repeat = 1;
$result = $this->createSequence($seq_name);
if (DB::isError($result)) {
return $this->raiseError($result);
}
} else {
$repeat = 0;
}
} while ($repeat);
if (DB::isError($result)) {
return $this->raiseError($result);
}
$arr = $result->fetchRow(DB_FETCHMODE_ORDERED);
return $arr[0];
}

//    
 //                     

function createSequence($seq_name)
{
$seqname = $this->getSequenceName($seq_name);
return $this->query("CREATE SEQUENCE ${seqname}");
}

//    
 //                   

function dropSequence($seq_name)
{
$seqname = $this->getSequenceName($seq_name);
return $this->query("DROP SEQUENCE ${seqname}");
}

//    
 //                     

function oci8RaiseError($errno = null)
{
if ($errno === null) {
$error = @OCIError($this->connection);
return $this->raiseError($this->errorCode($error['code']),
null, null, null, $error['message']);
} elseif (is_resource($errno)) {
$error = @OCIError($errno);
return $this->raiseError($this->errorCode($error['code']),
null, null, null, $error['message']);
}
return $this->raiseError($this->errorCode($errno));
}

//    
 //                      

/**
    *                                                  
    *                                                           
    *                                    
    */
function getSpecialQuery($type)
{
switch ($type) {
case 'tables':
$sql = "SELECT table_name FROM user_tables";
break;
default:
return null;
}
return $sql;
}

//    

function tableInfo($result, $mode = null)
{
$count = 0;
$res   = array();
/*
         *                                                           
         *
         *                           
         *        
         *                                
         *                                
         *                                
         *                                  
         *                                                 
         *                                               
         *                                         
         *
         *                             
         *        
         *                                      
         *                                
         *                                
         *                                
         *                                  
         *                                                 
         *                                               
         *                                         
         *                                                          
         *                                                                
         *                                                       
         *
         *                                  
         *                                       
         *                                                        
         *                        
         *
         *                                                        
         *                                      *                   
         *                                   
         *
         *                                            
         *                                                       
         *                              *                          
         */

//                                                         
 if (is_string($result)) {
$result = strtoupper($result);
$q_fields = "select column_name, data_type, data_length, data_precision,
                         nullable, data_default from user_tab_columns
                         where table_name='$result' order by column_id";
if (!$stmt = OCIParse($this->connection, $q_fields)) {
return $this->oci8RaiseError();
}
if (!OCIExecute($stmt, OCI_DEFAULT)) {
return $this->oci8RaiseError($stmt);
}
while (OCIFetch($stmt)) {
$res[$count]['table']       = $result;
$res[$count]['name']        = @OCIResult($stmt, 1);
$res[$count]['type']        = @OCIResult($stmt, 2);
$res[$count]['len']         = @OCIResult($stmt, 3);
$res[$count]['format']      = @OCIResult($stmt, 4);
$res[$count]['nullable']    = (@OCIResult($stmt, 5) == 'Y') ? true : false;
$res[$count]['default']     = @OCIResult($stmt, 6);
if ($mode & DB_TABLEINFO_ORDER) {
$res['order'][$res[$count]['name']] = $count;
}
if ($mode & DB_TABLEINFO_ORDERTABLE) {
$res['ordertable'][$res[$count]['table']][$res[$count]['name']] = $count;
}
$count++;
}
$res['num_fields'] = $count;
@OCIFreeStatement($stmt);
} else { //                                           
 if ($result === $this->last_stmt) {
$count = @OCINumCols($result);
for ($i=0; $i<$count; $i++) {
$res[$i]['name']  = @OCIColumnName($result, $i+1);
$res[$i]['type']  = @OCIColumnType($result, $i+1);
$res[$i]['len']   = @OCIColumnSize($result, $i+1);

$q_fields = "select table_name, data_precision, nullable, data_default from user_tab_columns where column_name='".$res[$i]['name']."'";
if (!$stmt = OCIParse($this->connection, $q_fields)) {
return $this->oci8RaiseError();
}
if (!OCIExecute($stmt, OCI_DEFAULT)) {
return $this->oci8RaiseError($stmt);
}
OCIFetch($stmt);
$res[$i]['table']       = OCIResult($stmt, 1);
$res[$i]['format']      = OCIResult($stmt, 2);
$res[$i]['nullable']    = (OCIResult($stmt, 3) == 'Y') ? true : false;
$res[$i]['default']     = OCIResult($stmt, 4);
OCIFreeStatement($stmt);

if ($mode & DB_TABLEINFO_ORDER) {
$res['order'][$res[$i]['name']] = $i;
}
if ($mode & DB_TABLEINFO_ORDERTABLE) {
$res['ordertable'][$res[$i]['table']][$res[$i]['name']] = $i;
}
}
$res['num_fields'] = $count;

} else {
return $this->raiseError(DB_ERROR_NOT_CAPABLE);
}
}
return $res;
}
}
//                 
//             
//                  
//     
?>
