<?php
//
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//        //           /       /                                           
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//
//                                                     
//
//                                                               
//           
//

//
//            
//                                               
//       //                  /       /                /       /     /       /                    
//
//                                                              
//                                    
//

require_once(INCLUDE_PATH."/common.php");

class DB_odbc extends DB_common
{
//               

var $connection;
var $phptype, $dbsyntax;
var $row = array();

//    
 //                

function DB_odbc()
{
$this->DB_common();
$this->phptype = 'odbc';
$this->dbsyntax = 'sql92';
$this->features = array(
'prepare' => true,
'pconnect' => true,
'transactions' => false,
'limit' => 'emulate'
);
$this->errorcode_map = array(
'01004' => DB_ERROR_TRUNCATED,
'07001' => DB_ERROR_MISMATCH,
'21S01' => DB_ERROR_MISMATCH,
'21S02' => DB_ERROR_MISMATCH,
'22003' => DB_ERROR_INVALID_NUMBER,
'22008' => DB_ERROR_INVALID_DATE,
'22012' => DB_ERROR_DIVZERO,
'23000' => DB_ERROR_CONSTRAINT,
'24000' => DB_ERROR_INVALID,
'34000' => DB_ERROR_INVALID,
'37000' => DB_ERROR_SYNTAX,
'42000' => DB_ERROR_SYNTAX,
'IM001' => DB_ERROR_UNSUPPORTED,
'S0000' => DB_ERROR_NOSUCHTABLE,
'S0001' => DB_ERROR_NOT_FOUND,
'S0002' => DB_ERROR_NOT_FOUND,
'S0011' => DB_ERROR_ALREADY_EXISTS,
'S0012' => DB_ERROR_NOT_FOUND,
'S0021' => DB_ERROR_ALREADY_EXISTS,
'S0022' => DB_ERROR_NOT_FOUND,
'S1009' => DB_ERROR_INVALID,
'S1090' => DB_ERROR_INVALID,
'S1C00' => DB_ERROR_NOT_CAPABLE
);
}

//    
 //              

/**
     *                                                        
     *
     *                                                               
     *                                                            
     *                     
     *
     *                                                         
     */
function connect($dsninfo, $persistent = false)
{
if (!DB::assertExtension('odbc'))
return $this->raiseError(DB_ERROR_EXTENSION_NOT_FOUND);

$this->dsn = $dsninfo;
if (!empty($dsninfo['dbsyntax'])) {
$this->dbsyntax = $dsninfo['dbsyntax'];
}
switch ($this->dbsyntax) {
case 'solid':
$this->features = array(
'prepare' => true,
'pconnect' => true,
'transactions' => true
);
$default_dsn = 'localhost';
break;
case 'navision':
//                                                        
 $this->features['limit'] = false;
break;
default:
break;
}
$dbhost = $dsninfo['hostspec'] ? $dsninfo['hostspec'] : 'localhost';
$user = $dsninfo['username'];
$pw = $dsninfo['password'];
if ($this->provides('pconnect')) {
$connect_function = $persistent ? 'odbc_pconnect' : 'odbc_connect';
} else {
$connect_function = 'odbc_connect';
}
$conn = @$connect_function($dbhost, $user, $pw);
if (!is_resource($conn)) {
return $this->raiseError(DB_ERROR_CONNECT_FAILED, null, null,
null, $this->errorNative());
}
$this->connection = $conn;
return DB_OK;
}

//    
 //                 

function disconnect()
{
$err = @odbc_close($this->connection);
$this->connection = null;
return $err;
}

//    
 //                  

/**
     *                                                               
     *            
     *
     *                            
     *
     *                                                              
     *                                                              
     *                        
     */
function simpleQuery($query)
{
$this->last_query = $query;
$query = $this->modifyQuery($query);
$result = @odbc_exec($this->connection, $query);
if (!$result) {
return $this->odbcRaiseError(); //             
 }
//                                                           
 //                                  
 if (DB::isManip($query)) {
$this->manip_result = $result; //                   
 return DB_OK;
}
$this->row[$result] = 0;
$this->manip_result = 0;
return $result;
}

//    
 //                 

/**
     *                                                                   
     *
     *                                     
     *
     *               
     *
     *                                                             
     */
function nextResult($result)
{
return odbc_next_result($result);
}

//    
 //               

function fetchRow($result, $fetchmode = DB_FETCHMODE_DEFAULT, $rownum=null)
{
if ($fetchmode == DB_FETCHMODE_DEFAULT) {
$fetchmode = $this->fetchmode;
}
$res = $this->fetchInto ($result, $arr, $fetchmode, $rownum);
if ($res !== DB_OK) {
return $res;
}
return $arr;
}

//    
 //                

function fetchInto($result, &$row, $fetchmode, $rownum=null)
{
$row = array();
if ($rownum !== null) {
$rownum++; //                    
 if (!function_exists('version_compare') || version_compare(phpversion(), "4.0.5", "lt")) {
$cols = odbc_fetch_into($result, $rownum, &$row);
} elseif (version_compare(phpversion(), '4.2.0', 'ge')) {
$cols = odbc_fetch_into($result, $row, $rownum);
} else {
$cols = odbc_fetch_into($result, $rownum, $row);
}
} else {
if (!function_exists('version_compare') || version_compare(phpversion(), "4.0.5", "lt")) {
$cols = odbc_fetch_into($result, &$row);
} else {
$cols = odbc_fetch_into($result, $row);
}
}

if (!$cols) {
/*                                                   
                                                       
                                                         
                                                 
             */
return null;
}
if ($fetchmode !== DB_FETCHMODE_ORDERED) {
for ($i = 0; $i < count($row); $i++) {
$colName = odbc_field_name($result, $i+1);
$a[$colName] = $row[$i];
}
$row = $a;
}
return DB_OK;
}

//    
 //                 

function freeResult($result)
{
if (is_resource($result)) {
//                   
 return odbc_free_result($result);
}
if (!isset($this->prepare_tokens[(int)$result])) {
return false;
}
unset($this->prepare_tokens[(int)$result]);
unset($this->prepare_types[(int)$result]);
return true;
}

//    
 //              

function numCols($result)
{
$cols = @odbc_num_fields($result);
if (!$cols) {
return $this->odbcRaiseError();
}
return $cols;
}

//    
 //                   

/**
    *                                                            
    *                         
    *                                                             
    *                                
    */
function affectedRows()
{
if (empty($this->manip_result)) {  //                       
 return 0;
}
$nrows = odbc_num_rows($this->manip_result);
if ($nrows == -1) {
return $this->odbcRaiseError();
}
return $nrows;
}

//    
 //              

/**
     *                                                               
     *         
     *
     *                                        
     *                                 
     */
function numRows($result)
{
$nrows = odbc_num_rows($result);
if ($nrows == -1) {
return $this->odbcRaiseError(DB_ERROR_UNSUPPORTED);
}
return $nrows;
}

//    
 //                  

/**
     *                                                          
     *                                   
     *
     *               
     *
     *                            
     */

function errorNative()
{
if (!isset($this->connection) || !is_resource($this->connection)) {
return odbc_error() . ' ' . odbc_errormsg();
}
return odbc_error($this->connection) . ' ' . odbc_errormsg($this->connection);
}

//    
 //             

/**
     *                                                        
     *                                                         
     *
     *               
     *
     *                                          
     *
     *                                                                
     *                  
     *
     *                                          
     */
function nextId($seq_name, $ondemand = true)
{
$seqname = $this->getSequenceName($seq_name);
$repeat = 0;
do {
$result = $this->query("update ${seqname} set id = id + 1");
if ($ondemand && DB::isError($result) &&
$result->getCode() == DB_ERROR_NOT_FOUND) {
$repeat = 1;
$result = $this->createSequence($seq_name);
if (DB::isError($result)) {
return $result;
}
$result = $this->query("insert into ${seqname} (id) values(0)");
} else {
$repeat = 0;
}
} while ($repeat);

if (DB::isError($result)) {
return $result;
}

$result = $this->query("select id from ${seqname}");
if (DB::isError($result)) {
return $result;
}

$row = $result->fetchRow(DB_FETCHMODE_ASSOC);
if (DB::isError($row || !$row)) {
return $row;
}

return $row['id'];
}

//    
 //                     

function createSequence($seq_name)
{
$seqname = $this->getSequenceName($seq_name);
return $this->query("CREATE TABLE ${seqname} ".
'(id bigint NOT NULL,'.
' PRIMARY KEY(id))');
}

//    
 //                   

function dropSequence($seq_name)
{
$seqname = $this->getSequenceName($seq_name);
return $this->query("DROP TABLE ${seqname}");
}

//    
 //                 

function autoCommit($onoff = false)
{
if (!@odbc_autocommit($this->connection, $onoff)) {
return $this->odbcRaiseError();
}
return DB_OK;
}

//    
 //             

function commit()
{
if (!@odbc_commit($this->connection)) {
return $this->odbcRaiseError();
}
return DB_OK;
}

//    
 //               

function rollback()
{
if (!@odbc_rollback($this->connection)) {
return $this->odbcRaiseError();
}
return DB_OK;
}

//    
 //                     

function odbcRaiseError($errno = null)
{
if ($errno === null) {
$errno = $this->errorCode(odbc_error($this->connection));
}
return $this->raiseError($errno, null, null, null,
$this->errorNative());
}

//    
 //                      

/**
    *                                                  
    *                                                           
    *                                    
    */
function getSpecialQuery($type)
{
switch ($type) {
case 'tables':
default:
return null;
}
return $sql;
}

//    

}

//                 
//             
//                  
//     
?>
